#!/usr/bin/python2
# -*- encoding:utf-8 -*-

"""Command line interface for Slidewall"""

# Import modules
#from sys import argv
from os import environ
import argparse
import slidewall
import gui
import conf


class Cli():
    
    p = None        # istanza di slidewall
    op = None        # istanza di argparse

    def __init__(self, name=None):
        self.op = argparse.ArgumentParser(description="Slidewall - CLI")
        self.op.add_argument('-n', '--name')
        self.op.add_argument('-t', '--hold-time')
        self.op.add_argument('-d', '--dest-dir')
        self.op.add_argument('-D', '--add-dir')
        self.op.add_argument('-T', '--add-by-tag')
        self.op.add_argument('-x', '--extract-xml')
        ##self.op.add_argument('-e', '--extract-xml') #extract xml file list as input list
        self.op.add_argument('--cup', action='store_true')
        self.op.add_argument('--cap', action='store_true')
        self.op.add_argument('-l', '--list', action='store_true')
        self.op.add_argument('-g', '--gui', action='store_true')
        self.op.add_argument('-cp', '--copy', action='store_true')
        self.op.add_argument('-s', '--simulate', action='store_true')
        self.op.add_argument('-v', '--verbose', action='store_true')
        self.op.add_argument('files', nargs='*')
        parsing = self.op.parse_args()
        self.d = vars(parsing)
        #print(self.d)          # Debug
        check = [ self.d[x] for x in self.d.keys() if (self.d[x] and (x=='files' or x=='extract_xml' or x=='add_by_tag' or x=='add_dir' or x=='gui')) ]
        if not check: self.op.error("Cannot run without arguments or input files!")
        #print(self.d)           # Debug
        
        
        if self.d['name']: 
            self.p = slidewall.Slidewall(self.d['name'])
        elif self.d['extract_xml']:
            fname = self.d['extract_xml'].split('/')[-1]
            name = fname.split('.')[0] #da sistemare
            self.p = slidewall.Slidewall(name)
        else: 
            self.p = slidewall.Slidewall('New_Slideshow')
            
        if self.d['files']:
            # Dobbiamo far risultare il path assoluto dei files
            # quindi davanti ci mettiamo pwd
            pwd = environ['PWD']
            if pwd[-1]!='/':
                pwd = pwd+'/'
            local_file_list = [pwd+f for f in self.d['files'] if f[0]!='/']
            local_file_list.extend([f for f in self.d['files'] if f[0]=='/'])
            self.d['files'] = local_file_list
            self.p.files = self.d['files']
            
        if self.d['add_dir']: self.p.directories = self.d['add_dir'].split(',')
        if self.d['add_by_tag']: self.p.tag_list = self.d['add_by_tag'].split(',') # i tag sono separati da , senza spazi
        if self.d['dest_dir']: self.p.dest_dir = self.d['dest_dir']
        if self.d['hold_time']: self.p.hold = self.d['hold_time']
        if self.d['extract_xml']: self.p.r_xml_file = self.d['extract_xml']
        ##if self.d['extract_xml']: self.p.r_xml_file = self.d['extract_xml']
        
        s = self.d['simulate']
        self.preload()

        if self.d['gui']:
            self.pprint("Opening GUI interface... :)")
            g = gui.Gui(self.p)
            g.start()
        else:
            if s: self.pprint("Starting simulation, it WILL NOT write anything down to disk!")
            self.write_Down(s)
        
    def preload(self):
        if self.d['verbose']: self.verbose()
        
        if self.p.r_xml_file:
            self.pprint("Extracting from %s ..." %(self.p.r_xml_file))
            self.p.read_xml()
            if self.d['verbose']: self.verbose()

        if self.p.directories:
            self.pprint("Adding from %s ..." %(str(self.p.directories)))
            self.p.dirs2files()
            if self.d['verbose']: self.verbose()
            
        if self.p.tag_list:
            self.pprint("Adding from tags ... it may take a while ...")
            self.p.tag_preload()
            if self.d['cup']: mode='cup'
            elif self.d['cap']: mode='cap'
            self.p.add_tagged(mode)
            if self.d['verbose']: self.verbose()
        
        if self.d['list']:
            self.pprint("File list:")
            if self.p.files:
                for f in self.p.files:
                    self.pprint("\t%s" %(f))
            else:
                self.pprint("Empty list...", i=35)

    def write_Down(self, s):
        self.pprint("Making destination directory %s ..." %(self.p.dest_dir))
        if not s: self.p.mkdir()
        if self.d['copy']:
            self.pprint("Copying files ...")
            if not s: self.p.cp_files()
        else:
            self.pprint("Linking files ...")
            if not s: self.p.ln_files()
        self.pprint("Building xml ...")
        if not s: self.p.make_xml()
        self.pprint("... and writing it down!")
        if not s: self.p.write_xml()
        self.pprint("Done! :)")
    
    def pprint(self, string, i=32):
        lst = string.split()
        string = ""
        tmp = []
        for y in self.d.keys():
            tmp.extend(y.split('_'))
        tmp = [ x for x in tmp if x not in ['to', 'by'] ]
        for s in lst:
            if s == "...": j = 1
            elif s in tmp: j = 31
            elif 'ing' in s: j = 34
            elif '/' in s: j = 35
            #elif match(r"\d+\.\d+\.\d+\.\d+", s): j = 35
            else: j = i
            string += "\033[%d;1m%s\033[0m " % (j, s)
        print(string)

    def verbose(self):
        self.pprint("File_list: %s" %(str(self.p.files)))


if __name__ == '__main__':
    Cli()
