#!/usr/bin/python2
# -*- encoding:utf-8 -*-

# System Modules
import subprocess, shlex
import argparse
from time import sleep
import sys
import signal
from random import shuffle

from xml.dom import minidom

# Local Modules
import conf
from xmlmanage import *

class SlidewallDaemon():

    xml_file = ""
    files = []
    picture_options = 'scaled'
    hold = 3000
    
    d = {}
    old_wallpaper = {'img' : "", 'options' : ""}

    def __init__(self):
        # INIT section:
        self.op = argparse.ArgumentParser(description="SlidewallDaemon")
        self.op.add_argument('-t', '--hold-time')
        self.op.add_argument('-r', '--random', action='store_true')
        self.op.add_argument('-v', '--verbose', action='store_true')
        self.op.add_argument('xml_file', nargs=1)
        parsing = self.op.parse_args()
        self.d = vars(parsing)
        if self.d['verbose']: print(self.d)          # Debug
        
        self.xml_file = self.d['xml_file'][0]
        self.get_old_wallpaper()
        if self.d['verbose']: print(self.old_wallpaper)          # Debug
        signal.signal(signal.SIGINT, self.sigINT_handler)
        signal.signal(signal.SIGTERM, self.sigINT_handler)
        signal.signal(signal.SIGQUIT, self.sigQUIT_handler)
        self.Run()
        
    def Run(self):
        # RUN section:
        n = 1
        i = 0
        while True:
            if i%n == 0:        # ogni giro di tutte le immagini rilegge il file, tante volte fosse stato aggiornato
                self.read_files()
                n = len(self.files)
                self.read_hold()
                if self.d['random']: shuffle(self.files) # shuffle the images list
                if self.d['verbose']: self.verbose()
            self.set_wallpaper(self.files[i%n], self.picture_options)
            self.wait()
            i += 1
    
    def sigQUIT_handler(self, signal, frame):
        print("\nSIGQUIT caught...nice! :)")

    def sigINT_handler(self, signal, frame):
        print("\nSIGINT caught!!!\nResetting old wallpaper and exiting...")
        self.set_wallpaper(self.old_wallpaper["img"], self.old_wallpaper["options"])
        sys.exit(0)
    
    def read_xml(self, tag):
        xmldoc = minidom.parse(self.xml_file)
        f_elements = xmldoc.getElementsByTagName(tag) # possibili tags 'file' 'duration' (?)
        values = map(lambda x: x.firstChild.nodeValue, f_elements)
        self.tmp = values
        
    def read_files(self):
        self.read_xml("file")
        self.files = self.tmp
    
    def read_hold(self):
        if self.d['hold_time']: # overread hold time from commandline
            self.hold = float(self.d['hold_time'])
        else:
            self.read_xml("duration")
            n = float(len(self.tmp))
            s = 0
            for x in self.tmp:
                s += float(x)
                self.hold = s/n
    
    def get_old_wallpaper(self):
        cmd = ['gsettings get org.gnome.desktop.background picture-uri', 'gsettings get org.gnome.desktop.background picture-options']
        key_list = ['img', 'options']
        cmd_lst = map(lambda x: shlex.split(x), cmd)
        i = 0
        for c in cmd_lst:
            p = subprocess.Popen(c, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (out, err) = p.communicate()
            if err: print(err)
            if out:
                out = out.replace("file://", '')
                out = out.replace('\n', '')
                out = out.replace("'", '')
                k = key_list[i]
                self.old_wallpaper[k] = out
            i += 1
    
    def set_wallpaper(self, img, opt):
        # gsettings set org.gnome.desktop.background picture-uri $uri
        cmd = [ 'gsettings set org.gnome.desktop.background picture-uri file://%s' %(img.encode("utf-8")),
               'gsettings set org.gnome.desktop.background picture-options %s' %(opt) ]
        cmd_lst = map(lambda x: shlex.split(x), cmd)
        for c in cmd_lst:
            p = subprocess.Popen(c, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (out, err) = p.communicate()
            if err: print err
    
    def wait(self):
        sleep(self.hold)
    
    def verbose(self):
        print("File_list: %s" %(str(self.files)))
        print("Hold_time: %s" %(str(self.hold)))
        

# --

if __name__ == '__main__':
    p = SlidewallDaemon()

# EOF
