#!/usr/bin/python2
# -*- encoding:utf-8 -*-

# System Modules
import subprocess
import re
from sys import argv
from os import environ, uname

#import os.path        # per implementare se esistono già dei files o dirs

from xml.dom import minidom
from xml.sax.saxutils import quoteattr

#Temp fix for ubuntu prettyprint
#if "Ubuntu" in uname()[3]:
#    import sys
#    sys.path.append('/usr/lib/python%s/site-packages/oldxml' % sys.version[:3])
#from xml.dom.ext import PrettyPrint
#from prettyprint import PrettyPrint

# Local Modules
import conf
from xmlmanage import *
from tagfilter import *

class Slidewall():
    """Main slidewall class"""
    
    # Nome dello slideshow
    name = ''
    # Path dei files o della dir da importare
    files = []
    directories = []
    # Xml file dello slideshow
    xml_file = ''
    r_xml_file = ''
    # File di configurazione di gnome per gli sfondi
    g_config_file = ''
    # Directory di destinazione
    dest_dir = ''
    
    hold = ''
    transition = ''
    xmldoc = None
    filetypes = []
    typeregexp = ''
    
    tag_instance = None
    tag_list = []
    tag_dict = {} # Associa ad ogni tag la sua lista file
    
    def __init__(self, name):
        self.name = name
        if conf.defaults['dest_dir']:
            self.dest_dir = "%s/%s" %(conf.defaults['dest_dir'], self.name)
        else:
            self.dest_dir = "%s/%s" %(environ['HOME'], self.name)
        if conf.defaults['g_config_file']:
            self.g_config_file = conf.defaults['g_config_file']
        else:
            self.g_config_file = "%s/.gnome2/backgrounds.xml" %(environ['HOME'])
        if conf.defaults['hold']: self.hold = conf.defaults['hold']
        if conf.defaults['transition'] : self.transition = conf.defaults['transition']
        self.xml_file = "%s/%s.xml" %(self.dest_dir, self.name)
        if conf.defaults['filetypes']: self.filetypes = conf.defaults['filetypes']
        
    def set_name(self, name = None):
        """Set the slideshow name"""
        self.name = name
    
    def set_dest(self, dest = None):
        """Set the directory where the slideshow's xml file will be placed"""
        self.dest_dir = dest
        
    def set_xml(self, xml = None):
        """Set the path of the slideshow's xml file"""
        self.xml_file = xml
    
    def set_r_xml(self, xml = None):
        """Set the path of an xml file to load and edit"""
        self.r_xml_file = xml
    
    def add_file(self, filename):
        """Add a file to the slideshow if not already in the file list"""
        if filename not in self.files:
            self.files.append(filename)
            
    def rm_file(self, filename):
        """Remove a file from the slideshow"""
        if filename in self.files:
            self.files.pop(filename)
    
    def add_dir(self, directory):
        """Add a directory to the directory list, but don't update the file list"""
        self.directories.append(directory)
        
    def mkregexp(self):
        """Build the regexp to filter files when adding from directory (by file extension)"""
        reg = "\.%s" %(self.filetypes[0])
        for s in self.filetypes[1:]:
            reg = "%s|\.%s" %(reg, s)
        reg = r".*[%s]$" %(reg)
        self.typeregexp = re.compile(reg, re.IGNORECASE)
    
    def dirs2files(self):
        """Add the files of every added directory to the file list"""
        # TODO: Da rifare con os.walk!
        for d in self.directories:
            #f = subprocess.Popen(['find', d, '-type', 'f'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            #(out, err) = f.communicate()
            #if err: print err
            #out = out.split('\n')
            #out = out[2:-1]        # perchè sennò ci sarebbe un elemento vuoto in coda e il primo è la dir stessa
            out = []
            for root, subFolders, files in os.walk(d):
                for f in files:
                    out.append(os.path.join(root,f))
            self.mkregexp()
            filtered = [i for i in out if self.typeregexp.match(i)]
            #self.files.extend(filtered)
            for f in filtered: self.add_file(f)
            self.directories = []

    def cp_files(self):
        """Copy img files to the destination directory"""
        self.copied_files = [] #list of copies
        for s in self.files:
            cp = subprocess.Popen(['cp','-L', s, self.dest_dir], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (out, err) = cp.communicate()
            if err: print err
            else:
                self.copied_files.append("%s/%s" %(self.dest_dir,s.split('/')[-1]))
        self.files = self.copied_files
            
    def ln_files(self):
        """Link img files to the destination directory"""
        self.linked_files = [] #list of links
        for s in self.files:
            ln = subprocess.Popen(['ln', '-s', s, self.dest_dir], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (out, err) = ln.communicate()
            if err: print err
            else:
                self.linked_files.append("%s/%s" %(self.dest_dir,s.split('/')[-1]))
        self.files = self.linked_files

#Nuovo da xmlmanage
    def make_xml(self):
        """Build the xml code from the file list"""
        xmldoc = minidom.Document()
        bg = minidom.Element("background")
        bg.appendChild(Header())
        l = len(self.files)
        for i in range(l):
            step = StepFrame(self.hold, self.transition, self.files[i], self.files[(i+1)%l])
            step.appendToNode(bg)
        
        xmldoc.appendChild(bg)
        self.xmldoc = xmldoc

    def read_xml(self):
        """Import a file list from an xml file"""
        xmldoc = minidom.parse(self.r_xml_file)
        f_elements = xmldoc.getElementsByTagName('file')
        files = map(lambda x: x.firstChild.nodeValue, f_elements)
        for f in files: self.add_file(f)
        
    def write_xml(self):
        """Write the xml code to a file"""
        # f = open(self.xml_file, 'w')
        # #self.xmldoc.writexml(f, '', '', '', "utf8")
        # #
        # PrettyPrint(self.xmldoc, f, 'utf8') #old syntax
        # f.close()

        with open(self.xml_file, "w") as f:
            f.write(self.xmldoc.toprettyxml(indent="  "))
#
        
    def mkdir(self):
        p = subprocess.Popen(['mkdir', '-p', self.dest_dir], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        (out, err) = p.communicate()
        if err: print err
        
    def tag_preload(self, tag_list = None):        # quando si inseriscono i tag, lui precarica
        """Load filtered by tag imgs"""
        if not tag_list: tag_list = self.tag_list
        self.tag_instance = TagFilter(conf.photo_lib)
        self.tag_instance.tag_list = tag_list
        self.tag_instance.populate_lib()
        self.tag_instance.filter()
    
    def add_tagged(self, mode='cup'):        # poi li aggiunge alla lista
        """Add preloaded filtered by tag imgs to the file list. Parameter mode can be 'cup', 'cap' default:'cup' """
        if mode == "cup":
            self.tag_instance.cup()
        elif mode == "cap":
            self.tag_instance.cap()
        
        for f in self.tag_instance.filtered:
            self.add_file(f)

# EOF
